From 623ecf07dc1b215cbc98f5804d58b571a649e9ba Mon Sep 17 00:00:00 2001
From: JD Smith <93749+jdtsmith@users.noreply.github.com>
Date: Sat, 6 Jul 2024 09:22:33 -0400
Subject: [PATCH] Fix formatting of tables with thead/tfoot but no tbody

Correctly handle formatting of tables containing thead and/or tfoot, but
without any tbody, to prevent including thead/tfoot content twice within
the table's derived body (Bug#71685).
* lisp/net.shr.el (shr--fix-tbody): Omit thead/tfoot from implicit body
* test/lisp/net/shr-resources/table.html:
* test/lisp/net/shr-resources/table.txt:
Added table rendering test.
---
 lisp/net/shr.el                        | 5 +++--
 test/lisp/net/shr-resources/table.html | 7 +++++++
 test/lisp/net/shr-resources/table.txt  | 5 +++++
 3 files changed, 15 insertions(+), 2 deletions(-)
 create mode 100644 test/lisp/net/shr-resources/table.html
 create mode 100644 test/lisp/net/shr-resources/table.txt

diff --git a/lisp/net/shr.el b/lisp/net/shr.el
index 3dadcb9a09b..fb72ea6aa67 100644
--- a/lisp/net/shr.el
+++ b/lisp/net/shr.el
@@ -2261,8 +2261,9 @@ shr-table-body
 (defun shr--fix-tbody (tbody)
   (nconc (list 'tbody (dom-attributes tbody))
          (cl-loop for child in (dom-children tbody)
-                  collect (if (or (stringp child)
-                                  (not (eq (dom-tag child) 'tr)))
+		  for tag = (and (not (stringp child)) (dom-tag child))
+		  unless (or (eq tag 'thead) (eq tag 'tfoot))
+		  collect (if (not (eq tag 'tr))
                               (list 'tr nil (list 'td nil child))
                             child))))
 
diff --git a/test/lisp/net/shr-resources/table.html b/test/lisp/net/shr-resources/table.html
new file mode 100644
index 00000000000..c5e8875ac91
--- /dev/null
+++ b/test/lisp/net/shr-resources/table.html
@@ -0,0 +1,7 @@
+<table>
+<thead><tr><th>A</th><th>B</th></tr></thead>
+<tr><td>1</td><td>2</td></tr>
+<tr><td>3</td><td>4</td></tr>
+5678
+<tfoot><tr><th>A</th><th>B</th></tr></tfoot>
+</table>
diff --git a/test/lisp/net/shr-resources/table.txt b/test/lisp/net/shr-resources/table.txt
new file mode 100644
index 00000000000..70939effb63
--- /dev/null
+++ b/test/lisp/net/shr-resources/table.txt
@@ -0,0 +1,5 @@
+  A  B    
+  1  2    
+  3  4    
+  5678     
+  A  B    
-- 
2.43.0

